
# This file was generated by 'versioneer.py' (0.18) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2021-11-05T18:05:23+0800",
 "dirty": false,
 "error": null,
 "full-revisionid": "0e48433fe5b1f3a64b5db638ed51be290d8a204a",
 "version": "2.9.39.3"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
