# coding: utf-8
from sqlalchemy import Numeric, Column
from .fundamental_base_sql import FundamentalBase


class StkIncomeGen(FundamentalBase):
    revenue = Column(Numeric(18, 2))
    operating_revenue = Column(Numeric(18, 2))
    sales_discount = Column(Numeric(18, 2))
    net_operating_revenue = Column(Numeric(18, 2))
    total_expense = Column(Numeric(18, 2))
    cost_of_goods_sold = Column(Numeric(18, 2))
    sales_tax = Column(Numeric(18, 2))
    gross_profit = Column(Numeric(18, 2))
    other_operating_income = Column(Numeric(18, 2))
    inventory_shrinkage = Column(Numeric(18, 2))
    selling_expense = Column(Numeric(18, 2))
    operating_expense = Column(Numeric(18, 2))
    ga_expense = Column(Numeric(18, 2))
    financing_expense = Column(Numeric(18, 2))
    exchange_gains_or_losses = Column(Numeric(18, 2))
    asset_depreciation = Column(Numeric(18, 2))
    profit_from_operation = Column(Numeric(18, 2))
    investment_income = Column(Numeric(18, 2))
    subsidy_income = Column(Numeric(18, 2))
    non_operating_revenue = Column(Numeric(18, 2))
    pnl_adjustment = Column(Numeric(18, 2))
    non_operating_expense = Column(Numeric(18, 2))
    disposal_loss_on_asset = Column(Numeric(18, 2))
    non_operating_net_profit = Column(Numeric(18, 2))
    profit_before_tax = Column(Numeric(18, 2))
    income_tax = Column(Numeric(18, 2))
    profit_from_ma = Column(Numeric(18, 2))
    net_profit = Column(Numeric(18, 2))
    net_profit_parent_company = Column(Numeric(18, 2))
    retained_profit_at_beginning = Column(Numeric(18, 2))
    profit_available_for_distribution = Column(Numeric(18, 2))
    statutory_welfare_reserve = Column(Numeric(18, 2))
    staff_incentive_welfare_reserve = Column(Numeric(18, 2))
    enterprise_expansion_reserve = Column(Numeric(18, 2))
    profit_available_for_owner_distribution = Column(Numeric(18, 2))
    preferred_stock_dividends = Column(Numeric(18, 2))
    other_surplus_reserve = Column(Numeric(18, 2))
    ordinary_stock_dividends = Column(Numeric(18, 2))
    basic_earnings_per_share = Column(Numeric(18, 4))
    other_income = Column(Numeric(18, 2))
    total_income = Column(Numeric(18, 2))
    total_income_parent_company = Column(Numeric(18, 2))
    total_income_minority = Column(Numeric(18, 2))

    fair_value_change_income = Column(Numeric(18, 2))
    invest_income_associates = Column(Numeric(18, 2))
    interest_income = Column(Numeric(18, 2))
    interest_expense = Column(Numeric(18, 2))

    other_operating_revenue = Column(Numeric(19, 4))
    other_operating_cost = Column(Numeric(19, 4))
    asset_impairment = Column(Numeric(19, 4))

    r_n_d = Column(Numeric(19, 4))
    minority_profit = Column(Numeric(19, 4))
    non_recurring_pnl = Column(Numeric(19, 4))
    net_profit_deduct_non_recurring_pnl = Column(Numeric(19, 4))
