# coding: utf-8
from sqlalchemy import Numeric, Column
from .fundamental_base_sql import FundamentalBase


class StkBalaGen(FundamentalBase):
    cash = Column(Numeric(18, 2))
    financial_asset_held_for_trading = Column(Numeric(18, 2))
    cash_equivalent = Column(Numeric(18, 2))
    current_investment = Column(Numeric(18, 2))
    current_investment_reserve = Column(Numeric(18, 2))
    net_current_investment = Column(Numeric(18, 2))
    bill_receivable = Column(Numeric(18, 2))
    devidend_receivable = Column(Numeric(18, 2))
    interest_receivable = Column(Numeric(18, 2))
    accts_receivable = Column(Numeric(18, 2))
    other_accts_receivable = Column(Numeric(18, 2))
    bad_debt_reserve = Column(Numeric(18, 2))
    net_accts_receivable = Column(Numeric(18, 2))
    other_receivables = Column(Numeric(18, 2))
    prepayment = Column(Numeric(18, 2))
    subsidy_receivable = Column(Numeric(18, 2))
    prepaid_tax = Column(Numeric(18, 2))
    inventory = Column(Numeric(18, 2))
    inventory_depreciation_reserve = Column(Numeric(18, 2))
    net_inventory = Column(Numeric(18, 2))
    deferred_expense = Column(Numeric(18, 2))
    contract_work = Column(Numeric(18, 2))
    long_term_debt_due_one_year = Column(Numeric(18, 2))
    non_current_debt_due_one_year = Column(Numeric(18, 2))
    other_current_assets = Column(Numeric(18, 2))
    current_assets = Column(Numeric(18, 2))
    financial_asset_available_for_sale = Column(Numeric(18, 2))
    financial_asset_hold_to_maturity = Column(Numeric(18, 2))
    real_estate_investment = Column(Numeric(18, 2))
    long_term_equity_investment = Column(Numeric(18, 2))
    long_term_receivables = Column(Numeric(18, 2))
    long_term_debt_investment = Column(Numeric(18, 2))
    other_long_term_investment = Column(Numeric(18, 2))
    long_term_investment = Column(Numeric(18, 2))
    provision_long_term_investment = Column(Numeric(18, 2))
    net_long_term_equity_investment = Column(Numeric(18, 2))
    net_long_term_debt_investment = Column(Numeric(18, 2))
    net_long_term_investment = Column(Numeric(18, 2))
    cost_fixed_assets = Column(Numeric(18, 2))
    accumulated_depreciation = Column(Numeric(18, 2))
    net_val_fixed_assets = Column(Numeric(18, 2))
    depreciation_reserve = Column(Numeric(18, 2))
    net_fixed_assets = Column(Numeric(18, 2))
    engineer_material = Column(Numeric(18, 2))
    construction_in_progress = Column(Numeric(18, 2))
    fixed_asset_to_be_disposed = Column(Numeric(18, 2))
    capitalized_biological_assets = Column(Numeric(18, 2))
    oil_and_gas_assets = Column(Numeric(18, 2))
    total_fixed_assets = Column(Numeric(18, 2))
    intangible_assets = Column(Numeric(18, 2))
    impairment_intangible_assets = Column(Numeric(18, 2))
    goodwill = Column(Numeric(18, 2))
    deferred_charges = Column(Numeric(18, 2))
    long_term_deferred_expenses = Column(Numeric(18, 2))
    other_long_term_assets = Column(Numeric(18, 2))
    total_intangible_and_other_assets = Column(Numeric(18, 2))
    deferred_income_tax_assets = Column(Numeric(18, 2))
    other_non_current_assets = Column(Numeric(18, 2))
    non_current_assets = Column(Numeric(18, 2))
    total_assets = Column(Numeric(18, 2))
    short_term_loans = Column(Numeric(18, 2))
    financial_liabilities = Column(Numeric(18, 2))
    notes_payable = Column(Numeric(18, 2))
    accts_payable = Column(Numeric(18, 2))
    advance_from_customers = Column(Numeric(18, 2))
    proxy_sale_revenue = Column(Numeric(18, 2))
    payroll_payable = Column(Numeric(18, 2))
    walfare_payable = Column(Numeric(18, 2))
    dividend_payable = Column(Numeric(18, 2))
    tax_payable = Column(Numeric(18, 2))
    interest_payable = Column(Numeric(18, 2))
    other_fees_payable = Column(Numeric(18, 2))
    internal_accts_payable = Column(Numeric(18, 2))
    other_payable = Column(Numeric(18, 2))
    short_term_debt = Column(Numeric(18, 2))
    accrued_expense = Column(Numeric(18, 2))
    estimated_liabilities = Column(Numeric(18, 2))
    deferred_income = Column(Numeric(18, 2))
    long_term_liabilities_due_one_year = Column(Numeric(18, 2))
    other_current_liabilities = Column(Numeric(18, 2))
    current_liabilities = Column(Numeric(18, 2))
    long_term_loans = Column(Numeric(18, 2))
    bond_payable = Column(Numeric(18, 2))
    long_term_payable = Column(Numeric(18, 2))
    grants_received = Column(Numeric(18, 2))
    housing_revolving_funds = Column(Numeric(18, 2))
    other_long_term_liabilities = Column(Numeric(18, 2))
    long_term_liabilities = Column(Numeric(18, 2))
    deferred_income_tax_liabilities = Column(Numeric(18, 2))
    other_non_current_liabilities = Column(Numeric(18, 2))
    non_current_liabilities = Column(Numeric(18, 2))
    total_liabilities = Column(Numeric(18, 2))
    paid_in_capital = Column(Numeric(18, 2))
    invesment_refund = Column(Numeric(18, 2))
    capital_reserve = Column(Numeric(18, 2))
    surplus_reserve = Column(Numeric(18, 2))
    statutory_reserve = Column(Numeric(18, 2))
    unrealised_investment_loss = Column(Numeric(18, 2))
    undistributed_profit = Column(Numeric(18, 2))
    equity_parent_company = Column(Numeric(18, 2))
    total_equity = Column(Numeric(18, 2))
    minority_interest = Column(Numeric(18, 2))
    total_equity_and_liabilities = Column(Numeric(18, 2))
    provision = Column(Numeric(18, 2))
    deferred_revenue = Column(Numeric(18, 2))

    non_current_liability_due_one_year = Column(Numeric(18, 2))
    liability_prefer_stock = Column(Numeric(18, 2))
    equity_prefer_stock = Column(Numeric(18, 2))
    equity_preferred_stock = Column(Numeric(18, 2))

    unrealised_investment_losses = Column(Numeric(19, 4))
    income_tax_refund = Column(Numeric(19, 4))
    non_current_asset_due_one_year = Column(Numeric(19, 4))
    long_term_deferred_income = Column(Numeric(19, 4))
    dividend_receivable = Column(Numeric(19, 4))
