# coding: utf-8
from sqlalchemy import Numeric, Column
from .fundamental_base_sql import FundamentalBase


class AnaStkFinIdx(FundamentalBase):
    earnings_per_share = Column(Numeric(18, 4))
    fully_diluted_earnings_per_share = Column(Numeric(18, 4))
    diluted_earnings_per_share = Column(Numeric(18, 4))
    adjusted_earnings_per_share = Column(Numeric(18, 4))
    adjusted_fully_diluted_earnings_per_share = Column(Numeric(18, 4))
    adjusted_diluted_earnings_per_share = Column(Numeric(18, 4))
    book_value_per_share = Column(Numeric(18, 4))
    operating_cash_flow_per_share = Column(Numeric(18, 4))
    operating_total_revenue_per_share = Column(Numeric(18, 4))
    operating_revenue_per_share = Column(Numeric(18, 4))
    capital_reserve_per_share = Column(Numeric(18, 4))
    earned_reserve_per_share = Column(Numeric(18, 4))
    undistributed_profit_per_share = Column(Numeric(18, 4))
    retained_earnings_per_share = Column(Numeric(18, 4))
    cash_flow_from_operations_per_share = Column(Numeric(18, 4))
    ebit_per_share = Column(Numeric(18, 4))
    free_cash_flow_company_per_share = Column(Numeric(18, 4))
    free_cash_flow_equity_per_share = Column(Numeric(18, 4))
    dividend_per_share = Column(Numeric(18, 4))
    return_on_equity = Column(Numeric(18, 4))
    return_on_equity_weighted_average = Column(Numeric(18, 4))
    return_on_equity_diluted = Column(Numeric(18, 4))
    adjusted_return_on_equity_average = Column(Numeric(18, 4))
    adjusted_return_on_equity_weighted_average = Column(Numeric(18, 4))
    adjusted_return_on_equity_diluted = Column(Numeric(18, 4))
    return_on_asset = Column(Numeric(18, 4))
    return_on_asset_net_profit = Column(Numeric(18, 4))
    return_on_invested_capital = Column(Numeric(18, 4))
    annual_return_on_equity = Column(Numeric(18, 4))
    annual_return_on_asset = Column(Numeric(18, 4))
    annual_return_on_asset_net_profit = Column(Numeric(18, 4))
    net_profit_margin = Column(Numeric(18, 4))
    gross_profit_margin = Column(Numeric(18, 4))
    cost_to_sales = Column(Numeric(18, 4))
    net_profit_to_revenue = Column(Numeric(18, 4))
    profit_from_operation_to_revenue = Column(Numeric(18, 4))
    ebit_to_revenue = Column(Numeric(18, 4))
    expense_to_revenue = Column(Numeric(18, 4))
    operating_profit_to_profit_before_tax = Column(Numeric(18, 4))
    invesment_profit_to_profit_before_tax = Column(Numeric(18, 4))
    non_operating_profit_to_profit_before_tax = Column(Numeric(18, 4))
    income_tax_to_profit_before_tax = Column(Numeric(18, 4))
    adjusted_profit_to_total_profit = Column(Numeric(18, 4))
    debt_to_asset_ratio = Column(Numeric(18, 4))
    equity_multiplier = Column(Numeric(18, 4))
    current_asset_to_total_asset = Column(Numeric(18, 4))
    non_current_asset_to_total_asset = Column(Numeric(18, 4))
    interest_bearing_debt_to_capital = Column(Numeric(18, 4))
    current_debt_to_total_debt = Column(Numeric(18, 4))
    non_current_debt_to_total_debt = Column(Numeric(18, 4))
    current_ratio = Column(Numeric(18, 4))
    quick_ratio = Column(Numeric(18, 4))
    super_quick_ratio = Column(Numeric(18, 4))
    debt_to_equity_ratio = Column(Numeric(18, 4))
    equity_to_debt_ratio = Column(Numeric(18, 4))
    equity_to_interest_bearing_debt = Column(Numeric(18, 4))
    ebit_to_debt = Column(Numeric(18, 4))
    ocf_to_debt = Column(Numeric(18, 4))
    ocf_to_interest_bearing_debt = Column(Numeric(18, 4))
    ocf_to_current_ratio = Column(Numeric(18, 4))
    ocf_to_net_debt = Column(Numeric(18, 4))
    time_interest_earned_ratio = Column(Numeric(18, 4))
    long_term_debt_to_working_capital = Column(Numeric(18, 4))
    account_payable_turnover_rate = Column(Numeric(18, 4))
    account_payable_turnover_days = Column(Numeric(18, 4))
    account_receivable_turnover_days = Column(Numeric(18, 4))
    inventory_turnover = Column(Numeric(18, 4))
    account_receivable_turnover_rate = Column(Numeric(18, 4))
    current_asset_turnover = Column(Numeric(18, 4))
    fixed_asset_turnover = Column(Numeric(18, 4))
    total_asset_turnover = Column(Numeric(18, 4))
    inc_earnings_per_share = Column(Numeric(18, 4))
    inc_diluted_earnings_per_share = Column(Numeric(18, 4))
    inc_revenue = Column(Numeric(18, 4))
    inc_operating_revenue = Column(Numeric(18, 4))
    inc_gross_profit = Column(Numeric(18, 4))
    inc_profit_before_tax = Column(Numeric(18, 4))
    inc_net_profit = Column(Numeric(18, 4))
    inc_adjusted_net_profit = Column(Numeric(18, 4))
    inc_cash_from_operations = Column(Numeric(18, 4))
    inc_return_on_equity = Column(Numeric(18, 4))
    inc_book_per_share = Column(Numeric(18, 4))
    inc_total_asset = Column(Numeric(18, 4))
    du_return_on_equity = Column(Numeric(18, 4))
    du_equity_multiplier = Column(Numeric(18, 4))
    du_asset_turnover_ratio = Column(Numeric(18, 4))
    du_profit_margin = Column(Numeric(18, 4))
    du_return_on_sales = Column(Numeric(18, 4))
    non_recurring_profit_and_loss = Column(Numeric(18, 4))
    adjusted_net_profit = Column(Numeric(18, 4))
    ebit = Column(Numeric(18, 4))
    ebitda = Column(Numeric(18, 4))
    invested_capital = Column(Numeric(18, 4))
    working_capital = Column(Numeric(18, 4))
    net_working_capital = Column(Numeric(18, 4))
    retained_earnings = Column(Numeric(18, 4))
    interest_bearing_debt = Column(Numeric(18, 4))
    net_debt = Column(Numeric(18, 4))
    non_interest_bearing_current_debt = Column(Numeric(18, 4))
    non_interest_bearing_non_current_debt = Column(Numeric(18, 4))
    fcff = Column(Numeric(18, 4))
    fcfe = Column(Numeric(18, 4))
    depreciation_and_amortization = Column(Numeric(18, 4))
    ev = Column(Numeric(21, 4))
    ev_2 = Column(Numeric(21, 4))
    ev_to_ebit = Column(Numeric(18, 4))

    ev_to_ebitda = Column(Numeric(19, 4))
    tangible_assets = Column(Numeric(19, 4))
    tangible_asset_to_debt = Column(Numeric(19, 4))
    tangible_asset_to_interest_bearing_debt = Column(Numeric(19, 4))
