BALANCE_SHEET_FIELDS = [
    "deferred_expense",
    "dividend_payable",
    "short_term_debt",
    "financial_liabilities",
    "equity_parent_company",
    "tax_payable",
    "payroll_payable",
    "intangible_assets",
    "deferred_revenue",
    "total_assets",
    "depreciation_reserve",
    "accts_payable",
    "deferred_income_tax_liabilities",
    "other_fees_payable",
    "financial_asset_hold_to_maturity",
    "construction_in_progress",
    "goodwill",
    "other_payable",
    "other_payable_interest_dividend",
    "long_term_receivables",
    "accrued_expense",
    "capital_reserve",
    "accumulated_depreciation",
    "surplus_reserve",
    "real_estate_investment",
    "engineer_material",
    "minority_interest",
    "provision",
    "oil_and_gas_assets",
    "long_term_liabilities_due_one_year",
    "bond_payable",
    "net_fixed_assets",
    "cash_equivalent",
    "financial_asset_available_for_sale",
    "estimated_liabilities",
    "bad_debt_reserve",
    "other_non_current_liabilities",
    "non_current_liability_due_one_year",
    "non_current_assets",
    "long_term_deferred_expenses",
    "net_long_term_equity_investment",
    "bill_receivable",
    "non_current_asset_due_one_year",
    "total_equity_and_liabilities",
    "long_term_equity_investment",
    "total_liabilities",
    "prepaid_tax",
    "grants_received",
    "paid_in_capital",
    "housing_revolving_funds",
    "notes_payable",
    "long_term_payable",
    "other_non_current_assets",
    "interest_receivable",
    "impairment_intangible_assets",
    "advance_from_customers",
    "deferred_income_tax_assets",
    "other_current_assets",
    "fixed_asset_to_be_disposed",
    "financial_asset_held_for_trading",
    "short_term_loans",
    "inventory",
    "deferred_income",
    "interest_payable",
    "non_current_liabilities",
    "dividend_receivable",
    "other_current_liabilities",
    "current_assets",
    "long_term_loans",
    "capitalized_biological_assets",
    "current_liabilities",
    "contract_work",
    "total_equity",
    "prepayment",
    "net_accts_receivable",
    "undistributed_profit",
    "equity_preferred_stock",
    "other_accts_receivable",
    "other_equity_investment",
    "other_illiquidy_financial_assets",
    "life_reserve_receivable",
    "reinsurance_payable",
    "sub_issue_security_proceeds",
    "fixed_deposits",
    "insurance_receivable",
    "policy_dividend_payable",
    "mortgaged_loan",
    "comission_payable",
    "assets_hold_for_sale",
    "foreign_currency_converted_difference",
    "refundable_capital_deposits",
    "other_reserves",
    "loans_advances_to_customers",
    "perpetual_equity_debt",
    "debt_investment",
    "financial_lease_receivable",
    "uncertained_impairment_losses",
    "fund_providing",
    "unclaimed_reserve_receivable",
    "unearned_reserve_receivable",
    "preference_shares",
    "insurer_deposit_investment",
    "resale_financial_assets",
    "borrowings_capital",
    "other_composite_income",
    "deposits_from_interbank",
    "financial_lease_payable",
    "independent_account_assets",
    "bill_accts_receivable",
    "derivative_financial_liabilities",
    "derivative_financial_assets",
    "borrowings_from_central_banks",
    "buy_back_security_proceeds",
    "other_receivables_interest_dividend",
    "contract_assets",
    "reinsurance_receivable",
    "compensation_payable",
    "use_right_assets",
    "perpetual_bond",
    "loan_account_receivables",
    "health_reserve_receivable",
    "insurance_contract_reserve",
    "settlement_provision",
    "financial_receivable",
    "reinsurance_reserve_receivable",
    "other_liabilities",
    "unclaimed_indemnity_reserve",
    "treasury_stock",
    "subrogation_fee_receivable",
    "uncertained_premium_reserve",
    "client_provision",
    "refundable_deposits",
    "independent_account_liabilities",
    "liabilities_hold_for_sale",
    "life_insurance_reserve",
    "security_deposits_received",
    "trade_risk_allowances",
    "other_assets",
    "general_reserve",
    "other_equity_instruments",
    "deposits_of_interbank",
    "interbank_deposits",
    "deposits",
    "bill_accts_payable",
    "insurer_mortgage_loan",
    "specific_reserve",
    "lend_capital",
    "seat_costs",
    "client_deposits",
    "accrued_staff_costs",
    "health_insurance_reserve",
    "other_debt_investment",
    "precious_metals",
    "advance_insurance",
    "contract_liabilities",
    "lease_liabilities",
    "proxy_security_proceeds",
    "consumable_biological_assets",
    'raw_materials',
    'materials_in_transit',
    'development_product',
    'construction_product',
    'rental_development_product',
    'development_cost',
    'undeveloped_land',
    'commodity_stocks',
    'semi_finished_goods',
    'inprocess_goods',
    'finished_goods',
    'low_value_consumption_goods',
    'installment_sales',
    'goods_in_consignors',
    'goods_purchased',
    'fuel',
    'wrappage',
    'materials_in_consignors',
    'materials_purchased',
    'materials_cost_variance',
    'spare_parts',
    'flight_spare_parts',
    'expensive_turnover_parts',
    'common_equipment',
    'other_inventory',
    'total_fixed_assets',
    'total_construction_in_progress',
    'deposit_in_associate',
    'debt_assets',
    'debit_of_associate',
    'total_shares',
    'total_a_shares',
    'circulation_a_shares',
    'non_circulation_a_shares'
]

CASHFLOW_STATEMENT_FIELDS = [
    "exchange_rate_change_effect",
    "cash_paid_for_other_financing_activities",
    "cash_received_from_investors",
    "cash_from_operating_activities",
    "cash_paid_to_financing_activities",
    "fcff",
    "cash_paid_for_employee",
    "cash_paid_for_goods_and_services",
    "cash_from_other_operating_activities",
    "cash_received_from_disposal_of_asset",
    "cash_received_from_sales_of_goods",
    "deferred_expense_amortization",
    "cash_paid_for_other_investment_activities",
    "fcfe",
    "cash_flow_from_financing_activities",
    "cash_received_from_other_financing_activities",
    "cash_paid_for_dividend_and_interest",
    "cash_paid_for_debt",
    "fixed_asset_depreciation",
    "cash_paid_for_investment_activities",
    "cash_paid_for_asset",
    "cash_received_from_financing_activities",
    "cash_flow_from_investing_activities",
    "cash_received_from_investment_activities",
    "net_inc_cash_and_equivalents",
    "intangible_asset_amortization",
    "depreciation_and_amortization",
    "cash_paid_for_other_operation_activities",
    "cash_received_from_financial_institution_borrows",
    "cash_received_from_other_investment_activities",
    "cash_paid_to_acquire_investment",
    "cash_flow_from_operating_activities",
    "cash_equivalent_increase",
    "cash_paid_for_operation_activities",
    "cash_paid_for_taxes",
    "refunds_of_taxes",
    "assets_depreciation_reserves",
    "net_cash_deal_from_sub",
    "net_cash_payment_from_sub",
    "cash_received_from_interests_and_commissions",
    "cash_paid_for_orignal_insurance",
    "cash_paid_for_reinsurance",
    "net_increase_from_repurchasing_business",
    "net_increase_in_pledge_loans",
    "cash_received_from_disposal_of_investment",
    "cash_received_from_issuing_security",
    "cash_received_from_reinsurance",
    "cash_received_from_sub_issue_security",
    "net_increase_from_central_bank",
    "cash_received_from_minority_invest_subsidiaries",
    "net_increase_from_trading_financial_assets",
    "cash_received_from_investment",
    "net_increase_from_loans_and_advances",
    "cash_received_from_proxy_security",
    "net_increase_from_disposing_financial_assets",
    "end_period_cash_equivalent",
    "net_increase_from__financing_buy_back",
    "net_increase_from_insurer_deposit_investment",
    "cash_paid_for_comissions",
    "cash_received_from_original_insurance",
    "cash_received_from_issuing_equity_instruments",
    "dividends_paid_to_minority_by_subsidiaries",
    "net_increase_from_lending_capital",
    "cash_paid_for_policy_dividends",
    "net_increase_from_financial_institutions",
    "draw_back_canceled_loans",
    "other_effecting_cash_equivalent_items",
    "begin_period_cash_equivalent",
    "net_increase_from_central_bank_and_banks",
    "net_increase_from_operating_buy_back",
    "net_increase_from_investing_buy_back",
    "net_deposit_increase",
    "net_increase_from_other_financial_institutions",
]

INCOME_STATEMENT_FIELDS = [
    "total_income_minority",
    "ga_expense",
    "invest_income_associates",
    "profit_from_operation",
    "r_n_d",
    "total_income",
    "other_operating_revenue",
    "total_income_parent_company",
    "other_operating_cost",
    "revenue",
    "net_profit_parent_company",
    "non_operating_revenue",
    "selling_expense",
    "ebitda",
    "minority_profit",
    "disposal_loss_on_asset",
    "non_recurring_pnl",
    "exchange_gains_or_losses",
    "financing_expense",
    "interest_expense",
    "operating_revenue",
    "net_profit",
    "total_expense",
    "fair_value_change_income",
    "other_income",
    "asset_impairment",
    "non_operating_expense",
    "net_profit_deduct_non_recurring_pnl",
    "profit_before_tax",
    "sales_tax",
    "basic_earnings_per_share",
    "ebit",
    "investment_income",
    "income_tax",
    "gross_profit",
    "cost_of_goods_sold",
    "interest_income",
    "unrealised_investment_loss",
    "fully_diluted_earnings_per_share",
    "operating_expense",
    "o_n_a_expense",
    "credit_asset_impairment",
    "disposal_income_on_asset",
    "financing_interest_income",
    "other_debt_investment_change",
    "compensation_expense",
    "other_net_income",
    "reinsurance_income",
    "insurance_commission_expense",
    "corporate_credit_risk_change",
    "net_open_hedge_income",
    "other_effecting_net_profits_items",
    "remearsured_other_income",
    "other_revenue",
    "net_commission_income",
    "earned_premiums",
    "foreign_currency_statement_converted_difference",
    "classified_by_ownership",
    "other_income_classified_income_statement",
    "other_debt_investment_reserve",
    "discontinued_operation_net_profit",
    "premiums_income",
    "policy_dividend_payout",
    "other_income_equity_unclassified_income_statement",
    "sub_issue_security_income",
    "amortization_reinsurance_cost",
    "assets_reclassified_other_income",
    "commission_income",
    "continuous_operation_net_profit",
    "unearned_premium_reserve",
    "classified_by_continuity_operation",
    "other_effecting_total_profits_items",
    "net_interest_income",
    "cash_flow_hedging_effective_portion",
    "financial_asset_available_for_sale_change",
    "net_trust_income",
    "other_income_unclassified_income_statement",
    "other_income_parent_company",
    "other_income_equity_classified_income_statement",
    "net_proxy_security_income",
    "premium_reserve",
    "other_equity_instruments_change",
    "commission_expense",
    "refunded_premiums",
    "other_income_minority",
    "amortization_expense",
    "amortization_premium_reserve",
    "reinsurance_cost",
    "financial_asset_hold_to_maturity_change",
    "reinsurance",
    "others",
    "financing_interest_expense"
]

TTM_FIELDS = [
    "operating_expenseTTM",
    "operating_costTTM",
    "net_invest_cashflowTTM",
    "sale_service_render_cashTTM",
    "ebitTTM",
    "operating_revenueTTM",
    "operating_profitTTM",
    "ni_from_operatingTTM",
    "net_cashflowTTM",
    "net_finance_cashflowTTM",
    "net_operate_cashflowTTM",
    "operating_payoutTTM",
    "asset_impairment_lossTTM",
    "non_operating_net_incomeTTM",
    "administration_expenseTTM",
    "gross_profitTTM",
    "total_operating_costTTM",
    "financial_expenseTTM",
    "ni_from_value_changeTTM",
    "np_parent_company_ownersTTM",
    "net_profitTTM",
    "total_profitTTM",
    "total_operating_revenueTTM"]

MAINDATA_FIELDS = ["adjusted_net_profit"]

RND_EXPENDITURE_SHEET_FIELDS = [
    "expensed_rnd",
    "capitalized_rnd",
    "total_rnd",
    "rnd_to_revenue",
    "capitalized_rnd_ratio",
    "rnd_employees",
    "rnd_employees_ratio"]

FIELDS_LIST_EX = RND_EXPENDITURE_SHEET_FIELDS + MAINDATA_FIELDS + TTM_FIELDS + INCOME_STATEMENT_FIELDS + CASHFLOW_STATEMENT_FIELDS + BALANCE_SHEET_FIELDS
